@@ -19,68 +19,33 @@
  *
  */
 
-#ifndef _INTERNAL_H
-# define _INTERNAL_H
+#ifndef INTERNAL_H
+# define INTERNAL_H
 
 # if HAVE_CONFIG_H
-# include "config.h"
-# endif
-
-# ifdef STDC_HEADERS
-#  include <stdio.h>
-#  include <stdlib.h>
-#  include <stdarg.h>
-#  include <ctype.h>
-#  include <string.h>
+#  include "config.h"
 # endif
 
 /* Get specifications. */
 # include "gsasl.h"
 
-/* Mechanism function prototypes. */
-typedef int (*_Gsasl_init_function) (Gsasl * ctx);
-typedef void (*_Gsasl_done_function) (Gsasl * ctx);
-typedef int (*_Gsasl_start_function) (Gsasl_session * sctx, void **mech_data);
-typedef int (*_Gsasl_step_function) (Gsasl_session * sctx,
-				     void *mech_data,
-				     const char *input, size_t input_len,
-				     char *output, size_t * output_len);
-typedef int (*_Gsasl_finish_function) (Gsasl_session * sctx, void *mech_data);
-typedef int (*_Gsasl_code_function) (Gsasl_session * sctx,
-				     void *mech_data,
-				     const char *input, size_t input_len,
-				     char *output, size_t * output_len);
+/* Get malloc, free, ... */
+# include <stdlib.h>
 
-/* Collection of mechanism functions for either client or server. */
-struct _Gsasl_mechanism_functions
-{
-  _Gsasl_init_function init;
-  _Gsasl_done_function done;
-  _Gsasl_start_function start;
-  _Gsasl_step_function step;
-  _Gsasl_finish_function finish;
-  _Gsasl_code_function encode;
-  _Gsasl_code_function decode;
-};
-
-/* Information about a mechanism. */
-struct _Gsasl_mechanism
-{
-  const char *name;
-
-  struct _Gsasl_mechanism_functions client;
-  struct _Gsasl_mechanism_functions server;
-};
-typedef struct _Gsasl_mechanism _Gsasl_mechanism;
+/* Get strlen, strcpy, ... */
+# include <string.h>
 
 /* Main library handle. */
 struct Gsasl
 {
   size_t n_client_mechs;
-  _Gsasl_mechanism *client_mechs;
+  Gsasl_mechanism *client_mechs;
   size_t n_server_mechs;
-  _Gsasl_mechanism *server_mechs;
-  void *application_data;
+  Gsasl_mechanism *server_mechs;
+  /* Callback. */
+  Gsasl_callback_function cb;
+  void *application_hook;
+  /* Obsolete stuff. */
   Gsasl_client_callback_authorization_id cbc_authorization_id;
   Gsasl_client_callback_authentication_id cbc_authentication_id;
   Gsasl_client_callback_password cbc_password;
@@ -111,9 +76,21 @@ struct Gsasl_session
 {
   Gsasl *ctx;
   int clientp;
-  _Gsasl_mechanism *mech;
-  void *application_data;
+  Gsasl_mechanism *mech;
   void *mech_data;
+  /* Properties. */
+  char *anonymous_token;
+  char *authid;
+  char *authzid;
+  char *password;
+  char *passcode;
+  char *pin;
+  char *suggestedpin;
+  char *service;
+  char *hostname;
+  char *gssapi_display_name;
+  /* Obsolete stuff. */
+  void *application_data;
 };
 
-#endif /* _INTERNAL_H */
+#endif /* INTERNAL_H */
